makeBoxPlot.func <- function(chrTable, linkage.contigs)
{

	includedContigs <- unlist(linkage.contigs)
	includeLength <- as.numeric(
		width(chrTable[chrTable$name %in% includedContigs]))/10^6
	excludeLength <- 
		as.numeric(width(chrTable[!(chrTable$name %in% includedContigs)]))/10^6


 	sumInclude <- round(sum(includeLength), digits=1)
	sumExclude <- round(sum(excludeLength), digits=1)
	percentInclude <- round(sumInclude / (sumInclude+sumExclude) *100, digits=1)

	boxplot(includeLength, 
	excludeLength, 
	names=c(paste(sumInclude, ' Mb included (', percentInclude, '%)', sep="" ),
			paste(sumExclude, ' Mb excluded (', round(100 - percentInclude, digits=1), '%)', sep="") ),
	main='Length of contigs included in assembly vs excluded',
	ylab='Contig length (MB)')

}

# Copyright (c) 2014, Mark Hills & Kieran O'Neill
# All rights reserved.

# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

#    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


####################################################################################################
#' makeBoxPlot -- creates boxplot of contigs included in the analysis vs those excluded
#' @param chrTable  A GRanges object consisting of contigs and positions. A meta column called 'names' must be present with names in the same
#' format as those from linkage.contigs. Generated by makeChrTable().
#' @param linkage.contigs A list of clustered contigs, generated by clusterContigs()
#' @aliases makeBoxPlot makeBoxPlot-ChrTable-LinkageGroupList-method
#' @rdname makeBoxPlot
#' @example inst/examples/makeBoxPlot.R
#' @return a box plot of included and excluded contigs
#'
#' @export
#'
####################################################################################################


setMethod('makeBoxPlot',
		  signature = signature(chrTable='ChrTable', linkage.contigs='LinkageGroupList'),
		  definition = makeBoxPlot.func
		  )
